#!/usr/bin/perl
#
#	Program:	filmlist.pl
#	Author :	Harald Piske (www.fairymail.de)
#	Written:	March 2010
#

use strict;
use CGI qw(:standard);
use vars qw($base $mydir @video);

setup();
main();
exit;

#
#	pre-set some vars and read list of films from file
#
sub setup
{
	$base = "/video0";
	$mydir = $0 =~ m|^.*/| ? $& : './';
	streamin ('video.tbl', \@video);
	$video[0] = [qw(title folder date time size content type genre rating flags)] unless @video;
}

#
#	static javascript goes here. Add dynamic stuff in main().
#
sub script
{
	return q{
function g(e){return document.getElementById(e);}
function content(e,v)
{
	var o = g(e);
	var _tc = o.innerText === undefined;
	if (v !== undefined)
	{
		if (_tc)
		{
			o.textContent = v;
		}else{
			o.innerText = v;
		}
	}
	if (_tc)
	{
		return o.textContent;
	}
	return o.innerText;
}
function show(what)
{
	var cell = what.innerHTML.match(/<td[^>]*>(.|[\r\n])*?<\/td>/g);
	for (var i in cell)
	{
		content('f' + i, cell[i].replace(/&nbsp;/gi,'').match(/>(?!<\w)((.|[\r\n])*?)<\//)[1]);
	}
	g('detail').style.display = 'block';
}
function select (obj)
{
	var i;
	var chldn = obj.childNodes;
	for (i = 0; i < chldn.length; i++)
	{
		var child = chldn[i];
		obj = g ('f' + child.id);
		if (obj)
		{
			obj.value = child.innerHTML.replace('&lt;', '<');
		}
	}
	obj = document.getElementsByName ('dsbl');
	for (i = 0; i < obj.length; i++)
	{
		obj[i].disabled = false;
	}
}
function sort (who)
{
	g('secret').value = 'sort ' + who.innerHTML;
	g('form1').submit();
}
function fg (who)
{
	var genre = who.parentNode.innerHTML.match(/>(\S+)</)[1];
	g('secret').value = genre;
	if (genre != '(alle)')
	{
		g('genre1').checked = false;
	}
	g('form1').submit();
}
};}	# end of string literal, end of statement, end of function

#
#	instead of a "die" - dump something to browser
#
sub report
{
	my $result = dumper (@_);

	$result =~ s/</&lt;/g;
	print header,
		start_html,
		"<pre>$result</pre>",
		end_html;
	exit;
		
}

#
#	read an array (or a hash) from file
#
sub streamin
{
	my ($fina, $var, $hash) = @_;

	if (open DB, "$mydir$fina")
	{
		$hash = join '', <DB>;
		close DB;
	}
	%$var = eval "($hash);" if ref $var eq 'HASH';
	@$var = eval "($hash);" if ref $var eq 'ARRAY';
	return $@;
}

#
#	write a struct from a variable to a file
#
sub streamout
{
	my ($fina) = shift;
	open FILE, ">$mydir$fina" or return "can't write to $fina";
	print FILE dumper (@_);
	close FILE;
	return '';
}

#
#	stream a struct to a string
#
sub dumper
{
	return _streamout ('', @_) . "\n";
}

#
#	recursively stream a struct to a string
#
sub _streamout
{
	my ($tab, @var, $var, $result, $ref, $in, $key) = @_;

	$in = "\t";
	for (;;)
	{
		return $result unless @var;
		$var = shift @var;
		$result .= ",\n" if $result;
		$ref = ref $var;
		if ($ref eq 'HASH')
		{
			$result .= "{\n" if $tab;
			for (keys %$var)
			{
				$key = $_;
				$key =~ s/\\/\\\\/g;
				$key =~ s/'/\\'/g;
				$result .= "$tab$in'$key' => " . _streamout ("$tab$in", $$var{$_}) . ",\n";
			}
			$result .= "$tab$in}" if $tab;
			next;
		}
		if ($ref eq 'ARRAY')
		{
			$result .= "[\n" if $tab;
			$result .= "$tab$in" . _streamout ("$tab$in", $$var[$_]) . ",\n" for 0 .. $#$var;
			$result .= "$tab$in]" if $tab;
			next;
		}
		if ($ref eq 'SCALAR')
		{
			$result .= "\\" . _streamout ("$tab  ", $$var);
			next;
		}
		next if $ref;
		unless ($var =~ /^\d+(\.\d+)?$/)
		{
			$var =~ s/\\/\\\\/g;
			$var =~ s/'/\\'/g;
			$var = "'" . $var . "'";
		}
		$result .= $var;
	}
}

sub test
{
	report param('genre');
}

#
#	convert special chars in video names
#
sub nicename
{
	my ($name) = @_;
	$name =~ tr/_/ /;
	$name =~ s/#([\da-fA-F]{2})/chr hex $1/eg;
	$name =~ s{/(.*)/}{$1};
	return $name;
}

#
#	read all of /video0 folder and update @video array
#
sub refresh
{
	my ($dir, $table, $key, $path, $summary, $type, $genre, $f, $n, $d, $t, @files, %video);

#	find all 001.vdr files
	@files = split /\n/, `find $base -follow -name 001.vdr`;
=snip
# for testing, read from file instead
	$0 =~ /[^\/]+$/;
	open DIR, "$`test.txt" or report ("can't open dir file from $`");
	@files = <DIR>;
	close DIR;
=cut
#	generate list of keys - use title+date+time
	%video = map {my $v = $video[$_]; (join ('/', @$v[0,2,3]), $_);} (1 .. $#video);

#	erase folder info - film might have moved or deleted
	$$_[1] = '' for @video[1 .. $#video];

#	process all files present in /video0 folder
	for (@files)
	{
		if (!m{^($base(.*/)([^/]+)/([\d-]+)\.(\d\d)\.(\d\d)[\d.]+rec/)001.vdr\s*$})
		{
			next;	# should never get here - ignore incomprehensible UNC pattern
			report ($_);	# ... or die screaming :-)
		}
		($path, $f, $n, $d, $t) = ($1, $2, $3, $4, "$5:$6");
		$f = nicename $f;
		$n = nicename $n;
		$key = $video{"$n/$d/$t"};
		if ($key)
		{	#	that film has been there before - just update location info
			$video[$key][1] = $f;
			next;
		}

#	got a new film - read size of index.vdr and description from summary.vdr
		$_ = [$n, $f, $d, $t, -s $path . 'index.vdr'];
		$summary = '';
		if (open TXT, $path . 'summary.vdr')
		{
			$summary = join '', <TXT>;
			close TXT;
			$summary = '' if $summary =~ /^ProjectX\s+\d/m;
		}
		$type = '';
		$type = 'Doku' if $path =~ /\bDokus?\b/;
		$type = 'Movie' if $path =~ /\bKino\b/;
#	only allow certain buzzwords for genre, not generally first or second folder name or something
		$genre = $f =~ /\b(Comedy|Crime|Tiere|Dokus|Action|Drama|Thriller|Romance|Mystery|Fantasy|SciFi|Horror)\b/
			? $1 : '';
		push @video, [@$_, $summary, $type, $genre];
	}
	streamout ('video.tbl', \@video);
}

#
# sort videos according to which table headers were clicked
#
sub filmsort
{
	my ($order, @list, $desc, @o, @key, %maps) = @_;

#	first item of order field gives sort direction
	$desc = shift @$order;
#	generate list of column indexes from order field
	%maps = (
		datum => 2,
		titel => 0,
		genre => 7,
		verzeichnis => 1,
		);
	for (@$order)
	{
		push @o, $maps{$_};
		push @o, 3 if $_ eq 'datum'; # sorts both date and time
	}
#	generate list of film keys with these sort columns
	@key = ();
	for (@video)
	{
		push @key, join '~', @$_[@o];
	}
#	sort the (filtered) film list using the film keys
	@list = sort {$key[$a] cmp $key[$b]} @list;
	@list = reverse @list if $desc;
	return (@list);
}

#
#	generate a checkbox+label pair that allows clicking the label to change the state
#
sub checker
{
	my ($id, $name, $click, $label, $checked, $trtd, $cb) = @_;

	$cb = $trtd ? "<tr><td>" : '';
	$cb .= "<input id='$id' name='$name' type='checkbox' value='$label' onclick=\"$click\"";
	$cb .= ' checked' if $checked;
	$cb .= "> <div style='display: inline; cursor: pointer;' onclick=\"g('$id').click();\">$label</div>";
	$cb .= "</td></tr>" if $trtd;
	return "$cb\n";
}

#
#	same as above with simplified parameters
#
sub checker2
{
	return checker (@_[0,0,1,2], scalar param($_[0]), $_[3]);
}

#
#	compute approximate film duration from index file size
#
sub duration
{
	my ($size, $min, $hour) = @_;

	$min = int ($size / 12000);
	$hour = int ($min / 60);
	$min -= $hour * 60;
	return sprintf ("%d:%02d", $hour, $min);
}

##################
#				 #
#    build UI    #
#				 #
##################
sub main
{
	my ($script, $table, $list, $date, $style, $grey, $genre, $i, $content, $search, $txo, $filter, $duration,
		@list, @line, @cmd, @order, %genre, %g);

#	hidden field "secret" is used to communicate commands
	refresh() if param('secret') eq 'refresh';

	$script = script();
	@order = split / /, (param ('order') || 'desc datum');

#	generate list of all possible genres
	$g{$$_[7]} = 1 for @video[1 .. $#video];

#	get list of selected genres
	%genre = ();
	@list = param ('genre');
	$genre{$_} = 1 for @list;
	%genre = ('(alle)' => 1) if $genre{'(alle)'} || (!%genre && param ('secret') ne '(alle)');
	%genre = %g unless %genre;
	$g{'(alle)'} = 1;

#	do what you're told
	{ # switch - perl does not have a "real" one
		@cmd = split /\s+/, lc param('secret');
		if ($cmd[0] eq 'sort')
		{
			if ($cmd[1] eq $order[1])
			{
				$order[0] = $order[0] ? '' : 'desc';
			}elsif ($cmd[1] eq 'datum') {
				@order = qw(desc datum);
			}else{
				@order = grep $_ ne $cmd[1], @order;
				splice @order, 0, 1, '', $cmd[1];
			}
			param ('order', join ' ', @order);
			last;
		}
	} # end of switch-block
	param ('secret', '');

# generate list of films - start with all of them
	@list = (1 .. $#video);
	unless (param ('deleted'))
	{
# remove deleted files
		@list = grep $video[$_][1], @list;
	}
	param ('search', '') if param ('clear');
	$search = param ('search');
	$search = '' if $search =~ /^\**$/;
# search field
	if ($search)
	{
# if it's a regex in slashes, just use it
		unless (substr $search, 0, 1 eq '/')
		{
# else use 'easy' wildcards
			$search =~ s/\\/\\\\/g;
			$search =~ s/\./\\./g;
			$search =~ s/\*/.*/g;
			$search =~ s/\?/./g;
		}
		$search = qr/$search/si;
		if (param ('txo'))
		{
# search title only
			@list = grep {$video[$_][0] =~ $search} @list;
		}else{
# search all fields
			@list = grep {my $line = join "\n", @{$video[$_]}; $line =~ $search} @list;
		}
	}
	unless ($genre{'(alle)'})
	{
# filter by genre(s)
		@list = grep $genre{$video[$_][7]}, @list;
	}
# sort what's left
	@list = filmsort (\@order, @list);

#
# generate detail area
#
	$grey = " style='background: #666699; color: white;'";
	$content = <<HTML
	<div id='detail' style='display:none;'>
	<table>
	<tr>
		<td$grey>Datum</td>
		<td><div id='f0'></div></td>
	</tr><tr>
		<td$grey>Titel</td>
		<td><div id='f1'></div></td>
	</tr><tr>
		<td$grey>Genre</td>
		<td><div id='f2'></div></td>
	</tr><tr>
		<td$grey>Verzeichnis</td>
		<td><div id='f3'></div></td>
	</tr><tr>
		<td$grey>Beschreibung</td>
		<td><div id='f4'></div></td>
	</tr><tr>
		<td$grey>Dauer (ca.)</td>
		<td><div id='f5'></div></td>
	</tr><tr style='display:none'>
		<td$grey>Bewertung</td>
		<td><div id='f6'></div></td>
	</tr><tr>
		<td colspan='2' align='left'>
			<input type='button' value='close' onclick=\"g('detail').style.display='none';\">
		</td>
	</tr></table></div>
HTML
;

#
# generate filter area
#
	$genre = '';
	$i = 0;
	for (sort {lc $a cmp lc $b} keys %g)
	{
		next unless $_;
		$i++;
		$genre .= checker ("genre$i", 'genre', 'fg(this);', $_, $genre{$_}, 1);
	}
	$genre .= checker2 ('deleted', "g('form1').submit();", 'gelschte', 1);
	$filter = Tr([td([
		'Suche nach ',
		textfield ('search', '*', 35),
		' ' . submit ('go!'),
		checker2 ('txo', "", 'nur Titel '),
		submit ('clear'),
	])]);

#
# generate list area
#
	$video[0] = [qw(title folder date time size content type genre rating flags)] unless @video;
	$list = "<table border=0><thead><tr style='background: #666699; color: white;'>";
	for (qw(Datum Titel Genre Verzeichnis))
	{
		$list .= "<th style='cursor: pointer;' onclick='sort(this);'>$_</th>\n";
	}
	$list .= "</tr></thead><tbody style='height: 580px; overflow-x: hidden; overflow-y: auto'>";
	$grey = 0;
	for (@list)
	{
		@line = @{$video[$_]};
		$line[1] ||= "<font color='red'>gelscht</font>";
		$date = $line[2] =~ /(\d\d)-(\d\d)-(\d\d)/ ? "$3.$2.$1" : '00.00.00';
		$duration = duration ($line[4]);
		$style = "cursor: pointer;";
		$style .= " background: #d0d0d0;" if $grey;
		$grey = !$grey;
		$list .= <<HTML
		<tr style="$style" id='$_' onclick="show(this);">
		<td>$date&nbsp;&nbsp;$line[3]</td>
		<td>$line[0]</td>
		<td>$line[7]</td>
		<td>$line[1]</td>
		<td style='display: none;'>$line[5]</td>
		<td style='display: none;'>$duration</td>
		<td style='display: none;'>$line[8]</td>
		</tr>
HTML
;
}
	$list .= "</tbody></table>";
	$style = <<STYLE
		<style type='text/css'>
			body {font-family: Arial,sans-serif;}
			table tr td {padding: 2px 3px; vertical-align: top;}
		</style>
STYLE
;
	$table = "<tr><td>
		<input type='button' value='Refresh' onclick=\"g('secret').value='refresh';g('form1').submit();\" >
		</td></tr>";
	$table   = table ({-border => 0, -width => '100%'}, $genre . $table);
	$filter  = table ({-border => 0}, $filter);
	$content = <<HTML
		<table border='0' width='100%' ><tr>
			<td width='110px' rowspan='3'>$table</td>
			<td><div style='width: 800px;'>$content</div></td>
		</tr><tr>
			<td>$filter</td>
		</tr><tr>
			<td><div style='height: 700px; overflow: auto;'>$list</div></td>
		</tr></table>
HTML
;
	print header(),
		start_html(
			-title => 'LinVDR Liste der Aufzeichnungen',
			-script => $script,
			),
		$style,
		start_multipart_form ({-id => 'form1'}),
		"<input type='hidden' id='secret' name='secret' value='' />",
		hidden ('order', 'desc datum'),
		$content,
		end_form;
	print end_html;
}
